use crate::*;

#[::tracing_test::traced_test]
#[::tokio::test]
async fn test_connection_dirsync() {
    use enc::asym::{KeyID, PrivKey, PubKey};
    let rand = enc::Random::new();
    let (priv_exchange_key, pub_exchange_key) =
        match enc::asym::KeyExchangeKind::X25519DiffieHellman.new_keypair(&rand)
        {
            Ok((privkey, pubkey)) => {
                (PrivKey::Exchange(privkey), PubKey::Exchange(pubkey))
            }
            Err(_) => {
                assert!(false, "Can't generate random keypair");
                return;
            }
        };
    let cfg_client = {
        let mut cfg = config::Config::default();
        cfg.threads = Some(::core::num::NonZeroUsize::new(1).unwrap());
        cfg
    };
    let cfg_server = {
        let mut cfg = cfg_client.clone();
        cfg.keys = [(KeyID(42), priv_exchange_key, pub_exchange_key)].to_vec();
        cfg
    };

    let (server, mut srv_workers) =
        Fenrir::with_workers(&cfg_server).await.unwrap();

    let srv_worker = srv_workers.pop().unwrap();
    let local_thread = ::tokio::task::LocalSet::new();
    local_thread.spawn_local(async move { srv_worker.await });

    let (client, mut cli_workers) =
        Fenrir::with_workers(&cfg_client).await.unwrap();
    let cli_worker = cli_workers.pop().unwrap();
    local_thread.spawn_local(async move { cli_worker.await });

    use crate::{
        connection::handshake::HandshakeID,
        dnssec::{record, Record},
    };

    let port: u16 = server.addresses()[0].port();

    let dnssec_record = Record {
        public_keys: [(KeyID(42), pub_exchange_key)].to_vec(),
        addresses: [record::Address {
            ip: ::std::net::IpAddr::V4(::std::net::Ipv4Addr::new(127, 0, 0, 1)),
            port: Some(::core::num::NonZeroU16::new(port).unwrap()),
            priority: record::AddressPriority::P1,
            weight: record::AddressWeight::W1,
            handshake_ids: [HandshakeID::DirectorySynchronized].to_vec(),
            public_key_idx: [record::PubKeyIdx(0)].to_vec(),
        }]
        .to_vec(),
        key_exchanges: [enc::asym::KeyExchangeKind::X25519DiffieHellman]
            .to_vec(),
        hkdfs: [enc::hkdf::HkdfKind::Sha3].to_vec(),
        ciphers: [enc::sym::CipherKind::XChaCha20Poly1305].to_vec(),
    };

    server.graceful_stop().await;
    client.graceful_stop().await;
    return;

    let _ = client
        .connect_resolved(
            dnssec_record,
            &Domain("example.com".to_owned()),
            auth::SERVICEID_AUTH,
        )
        .await;
    server.graceful_stop().await;
    client.graceful_stop().await;
}
